@@ -1,33 +1,37 @@
 package Net::DNS::Parameters;
 
 #
-# $Id: Parameters.pm 1513 2016-11-12 03:20:58Z willem $
+# $Id: Parameters.pm 1527 2017-01-18 21:42:48Z willem $
 #
-use vars qw($VERSION);
-$VERSION = (qw$LastChangedRevision: 1513 $)[1];
+our $VERSION = (qw$LastChangedRevision: 1527 $)[1];
 
 
 ################################################
 ##
 ##	Domain Name System (DNS) Parameters
-##	(last updated 2016-10-18)
+##	(last updated 2016-12-09)
 ##
 ################################################
 
 
 use strict;
+use warnings;
 use integer;
 use Carp;
 
-use constant DNSEXTLANG => defined eval {
+
+our $DNSEXTLANG = 'ARPA.';		## draft-levine-dnsextlang
+
+use constant DNSEXTLANG => defined eval <<'END';
+	die 'preempt failure' if $^O =~ /cygwin|MSWin32/i;
+	require IO::File;
 	local $SIG{__WARN__} = sub { };
-	new FileHandle("RRTYPEgen |");
-};
+	new IO::File('RRTYPEgen |');
+END
 
 
 use base qw(Exporter);
-use vars qw(@EXPORT);
-@EXPORT = qw(
+our @EXPORT = qw(
 		classbyname classbyval %classbyname
 		typebyname typebyval %typebyname
 		opcodebyname opcodebyval
@@ -37,21 +41,19 @@ use vars qw(@EXPORT);
 
 
 # Registry: DNS CLASSes
-use vars qw( %classbyname %classbyval );
-%classbyname = (
+our %classbyname = (
 	IN   => 1,						# RFC1035
 	CH   => 3,						# Chaosnet
 	HS   => 4,						# Hesiod
 	NONE => 254,						# RFC2136
 	ANY  => 255,						# RFC1035
 	);
-%classbyval = reverse %classbyname;
+our %classbyval = reverse %classbyname;
 %classbyname = ( '*' => 255, %classbyname, map lc($_), %classbyname );
 
 
 # Registry: Resource Record (RR) TYPEs
-use vars qw( %typebyname %typebyval );
-%typebyname = (
+our %typebyname = (
 	A	   => 1,					# RFC1035
 	NS	   => 2,					# RFC1035
 	MD	   => 3,					# RFC1035
@@ -137,26 +139,24 @@ use vars qw( %typebyname %typebyval );
 	TA	   => 32768,					# http://cameo.library.cmu.edu/ http://www.watson.org/~weiler/INI1999-19.pdf
 	DLV	   => 32769,					# RFC4431
 	);
-%typebyval = reverse %typebyname;
+our %typebyval = reverse %typebyname;
 %typebyname = ( '*' => 255, %typebyname, map lc($_), %typebyname );
 
 
 # Registry: DNS OpCodes
-use vars qw( %opcodebyname %opcodebyval );
-%opcodebyname = (
+our %opcodebyname = (
 	QUERY  => 0,						# RFC1035
 	IQUERY => 1,						# RFC3425
 	STATUS => 2,						# RFC1035
 	NOTIFY => 4,						# RFC1996
 	UPDATE => 5,						# RFC2136
 	);
-%opcodebyval = reverse %opcodebyname;
+our %opcodebyval = reverse %opcodebyname;
 %opcodebyname = ( NS_NOTIFY_OP => 4, %opcodebyname, map lc($_), %opcodebyname );
 
 
 # Registry: DNS RCODEs
-use vars qw( %rcodebyname %rcodebyval );
-%rcodebyname = (
+our %rcodebyname = (
 	NOERROR	  => 0,						# RFC1035
 	FORMERR	  => 1,						# RFC1035
 	SERVFAIL  => 2,						# RFC1035
@@ -179,13 +179,12 @@ use vars qw( %rcodebyname %rcodebyval );
 	BADTRUNC  => 22,					# RFC4635
 	BADCOOKIE => 23,					# RFC7873
 	);
-%rcodebyval = reverse( BADSIG => 16, %rcodebyname );
+our %rcodebyval = reverse( BADSIG => 16, %rcodebyname );
 %rcodebyname = ( %rcodebyname, map lc($_), %rcodebyname );
 
 
 # Registry: DNS EDNS0 Option Codes (OPT)
-use vars qw( %ednsoptionbyname %ednsoptionbyval );
-%ednsoptionbyname = (
+our %ednsoptionbyname = (
 	LLQ		=> 1,					# http://files.dns-sd.org/draft-sekar-dns-llq.txt
 	UL		=> 2,					# http://files.dns-sd.org/draft-sekar-dns-ul.txt
 	NSID		=> 3,					# RFC5001
@@ -198,14 +197,14 @@ use vars qw( %ednsoptionbyname %ednsoptionbyval );
 	'TCP-KEEPALIVE' => 11,					# RFC7828
 	PADDING		=> 12,					# RFC7830
 	CHAIN		=> 13,					# RFC7901
+	DEVICEID	=> 26946,				# https://docs.umbrella.com/developer/networkdevices-api/identifying-dns-traffic2
 	);
-%ednsoptionbyval = reverse %ednsoptionbyname;
+our %ednsoptionbyval = reverse %ednsoptionbyname;
 %ednsoptionbyname = ( %ednsoptionbyname, map lc($_), %ednsoptionbyname );
 
 
 # Registry: DNS Header Flags
-use vars qw( %dnsflagbyname );
-%dnsflagbyname = (
+our %dnsflagbyname = (
 	AA => 0x0400,						# RFC1035
 	TC => 0x0200,						# RFC1035
 	RD => 0x0100,						# RFC1035
@@ -213,13 +212,14 @@ use vars qw( %dnsflagbyname );
 	AD => 0x0020,						# RFC4035
 	CD => 0x0010,						# RFC4035
 	);
+%dnsflagbyname = ( %dnsflagbyname, map lc($_), %dnsflagbyname );
 
 
 # Registry: EDNS Header Flags (16 bits)
-use vars qw( %ednsflagbyname );
-%ednsflagbyname = (
+our %ednsflagbyname = (
 	DO => 0x8000,						# RFC4035 RFC3225
 	);
+%ednsflagbyname = ( %ednsflagbyname, map lc($_), %ednsflagbyname );
 
 
 ########
@@ -257,7 +257,7 @@ sub typebyname {
 			croak "typebyname( $name ) out of range" if $val > 0xffff;
 			return $val;
 		}
-		_typespec("$name.RRNAMES");
+		_typespec("$name.RRNAME");
 		return $typebyname{uc $name} || croak "unknown type $name";
 			}
 }
@@ -269,7 +269,7 @@ sub typebyval {
 		$val += 0;
 		croak "typebyval( $val ) out of range" if $val > 0xffff;
 		$typebyval{$val} = "TYPE$val";
-		_typespec("$val.RRTYPES");
+		_typespec("$val.RRTYPE");
 		return $typebyval{$val};
 			}
 }
@@ -332,18 +332,21 @@ sub register {				## register( 'TOY', 1234 )	(NOT part of published API)
 sub _typespec {				## draft-levine-dnsextlang
 	eval <<'END' if DNSEXTLANG;
 	my ($node) = @_;
-	my $repository = 'ARPA.';
 	require Net::DNS::Resolver;
 	my $resolver = new Net::DNS::Resolver;
-	my $response = $resolver->send( "$node.$repository", 'TXT' );
-	my ( $tag, $language, @stanza );
+	my $response = $resolver->send( "$node.$DNSEXTLANG", 'TXT' );
+
 	foreach my $txt ( grep $_->type eq 'TXT', $response->answer ) {
-		( $tag, $language, @stanza ) = $txt->txtdata;
-		next unless $tag =~ /^RRTYPE=\d+$/;
-		register( split /[:\s]/, $stanza[0] );
-		last if $language =~ /^EN/i;
+		my @stanza = $txt->txtdata;
+		my ( $tag, $identifier ) = @stanza;
+		next unless defined($tag) && $tag =~ /^RRTYPE=\d+$/;
+		register( split /[:\s]/, $identifier );
+		return unless defined wantarray;
+		require 5.008009;				# support for reference in @INC
+		my @arg = map { s/\s.*$//; qq("$_") } @stanza;	# strip descriptive text
+		return new IO::File("RRTYPEgen @arg |");
 	}
-	return @stanza;
+	return undef;
 END
 }
 
@@ -384,7 +387,7 @@ the given numerical code.
 
 =head1 COPYRIGHT
 
-Copyright (c)2012 Dick Franks
+Copyright (c)2012,2016 Dick Franks.
 
 Portions Copyright (c)1997 Michael Fuhr.
 
