@@ -1,13 +1,13 @@
 package Net::DNS::RR::OPT;
 
 #
-# $Id: OPT.pm 1474 2016-04-12 13:21:25Z willem $
+# $Id: OPT.pm 1528 2017-01-18 21:44:58Z willem $
 #
-use vars qw($VERSION);
-$VERSION = (qw$LastChangedRevision: 1474 $)[1];
+our $VERSION = (qw$LastChangedRevision: 1528 $)[1];
 
 
 use strict;
+use warnings;
 use base qw(Net::DNS::RR);
 
 =head1 NAME
