@@ -1,13 +1,13 @@
 package Net::DNS::RR::TSIG;
 
 #
-# $Id: TSIG.pm 1472 2016-04-04 21:49:18Z willem $
+# $Id: TSIG.pm 1528 2017-01-18 21:44:58Z willem $
 #
-use vars qw($VERSION);
-$VERSION = (qw$LastChangedRevision: 1472 $)[1];
+our $VERSION = (qw$LastChangedRevision: 1528 $)[1];
 
 
 use strict;
+use warnings;
 use base qw(Net::DNS::RR);
 
 =head1 NAME
@@ -229,8 +229,9 @@ sub macbin {
 
 sub prior_mac {
 	my $self = shift;
+	my @args = map { /[^0-9A-Fa-f]/ ? croak "corrupt hexadecimal" : $_ } @_;
 
-	$self->prior_macbin( pack "H*", map { die "!hex!" if m/[^0-9A-Fa-f]/; $_ } join "", @_ ) if scalar @_;
+	$self->prior_macbin( pack "H*", join "", @args ) if scalar @args;
 	unpack "H*", $self->prior_macbin() if defined wantarray;
 }
 
@@ -245,8 +246,9 @@ sub prior_macbin {
 
 sub request_mac {
 	my $self = shift;
+	my @args = map { /[^0-9A-Fa-f]/ ? croak "corrupt hexadecimal" : $_ } @_;
 
-	$self->request_macbin( pack "H*", map { die "!hex!" if m/[^0-9A-Fa-f]/; $_ } join "", @_ ) if scalar @_;
+	$self->request_macbin( pack "H*", join "", @args ) if scalar @args;
 	unpack "H*", $self->request_macbin() if defined wantarray;
 }
 
@@ -282,7 +284,7 @@ sub other {
 }
 
 
-sub other_data {&other}						# uncoverable pod
+sub other_data { &other; }					# uncoverable pod
 
 
 sub sig_function {
@@ -446,24 +448,19 @@ sub verify {
 		if ( $arg->isa('Net::DNS::Packet') ) {
 			my $request = $arg->sigrr;		# request TSIG
 			my $rqstkey = lc( join '+', $request->name, $request->algorithm );
-			unless ( $signerkey eq $rqstkey ) {
-				$self->error(17);
-				return;
-			}
+			$self->error(17) unless $signerkey eq $rqstkey;
 			$self->request_macbin( $request->macbin );
 
 		} elsif ( $arg->isa(__PACKAGE__) ) {
 			my $priorkey = lc( join '+', $arg->name, $arg->algorithm );
-			unless ( $signerkey eq $priorkey ) {
-				$self->error(17);
-				return;
-			}
+			$self->error(17) unless $signerkey eq $priorkey;
 			$self->prior_macbin( $arg->macbin );
 
 		} else {
 			croak 'Usage: $tsig->verify( $reply, $query )';
 		}
 	}
+	return if $self->{error};
 
 	my $sigdata = $self->sig_data($data);			# form data to be verified
 	my $tsigmac = $self->_mac_function($sigdata);
@@ -472,14 +469,10 @@ sub verify {
 	my $macbin = $self->macbin;
 	my $maclen = length $macbin;
 	my $minlen = length($tsigmac) >> 1;			# per RFC4635, 3.1
-	if ( $maclen < $minlen or $maclen < 10 or $maclen > length $tsigmac ) {
-		$self->error(1);
-		return;
-	}
+	$self->error(16) unless $macbin eq substr $tsigmac, 0, $maclen;
+	$self->error(1) if $maclen < $minlen or $maclen < 10 or $maclen > length $tsigmac;
 
-	return $tsig if $macbin eq substr $tsigmac, 0, $maclen;
-	$self->error(16);
-	return;
+	return $self->{error} ? undef : $tsig;
 }
 
 sub vrfyerrstr {
@@ -566,6 +559,7 @@ sub vrfyerrstr {
 
 sub _chain {
 	my $self = shift;
+	$self->{link} = undef;
 	bless {%$self, link => $self}, ref($self);
 }
 
@@ -851,6 +845,6 @@ DEALINGS IN THE SOFTWARE.
 
 L<perl>, L<Net::DNS>, L<Net::DNS::RR>, RFC2845, RFC4635
 
-<TSIG Algorithm Names|http://www.iana.org/assignments/tsig-algorithm-names>
+L<TSIG Algorithm Names|http://www.iana.org/assignments/tsig-algorithm-names>
 
 =cut
